@@ -1,4 +1,4 @@
-/*  $Id: xml_edit.c,v 1.40 2004/03/30 03:43:02 mgrouch Exp $  */
+/*  $Id: xml_edit.c,v 1.45 2005/01/08 00:07:03 mgrouch Exp $  */
 
 /*
 
@@ -104,7 +104,7 @@ int ops_count = 0;
 static XmlEdAction ops[MAX_XML_ED_OPS];
 
 /*
- * usage string chunk : 509 char min on ISO C90
+ * usage string chunk : 509 char max on ISO C90
  */
 static const char edit_usage_str_1[] =
 "XMLStarlet Toolkit: Edit XML document(s)\n"
@@ -154,11 +154,11 @@ edUsage(int argc, char **argv)
 {
     extern const char more_info[];
     FILE* o = stderr;
-    fprintf(o, edit_usage_str_1);
-    fprintf(o, edit_usage_str_2);
-    fprintf(o, edit_usage_str_3);
-    fprintf(o, edit_usage_str_4);
-    fprintf(o, more_info);
+    fprintf(o, "%s", edit_usage_str_1);
+    fprintf(o, "%s", edit_usage_str_2);
+    fprintf(o, "%s", edit_usage_str_3);
+    fprintf(o, "%s", edit_usage_str_4);
+    fprintf(o, "%s", more_info);
     exit(1);
 }
 
@@ -623,15 +623,22 @@ edDelete(xmlDocPtr doc, char *str)
                 /*
                 fprintf(stderr, "Set contains %d nodes:\n", cur->nodeNr);
                 */
-                for (i = 0; i < cur->nodeNr; i++)
+                /*for (i = 0; i < cur->nodeNr; i++)*/
+                for (i = cur->nodeNr - 1; i >= 0; i--)
                 {
                     /*
                      *  delete node
                      */
                     xmlUnlinkNode(cur->nodeTab[i]);
-                    /* xmlFreeNode(cur->nodeTab[i]); ??? */
+                    
+                    /*
+                     * Free node and children
+                     */
+                    xmlFreeNode(cur->nodeTab[i]);
+                    cur->nodeTab[i] = NULL;
                 }
             }
+            
             break;
         }
         default:
@@ -695,6 +702,8 @@ edMove(xmlDocPtr doc, char *from, char *to)
 #if defined(LIBXML_XPTR_ENABLED)
     }
 #endif
+
+    if (ctxt) xmlXPathFreeContext(ctxt);
     if (res == NULL) return;
 
     /********************************************************/
@@ -1052,7 +1061,13 @@ edMain(int argc, char **argv)
     {
         xmlDocPtr doc = xmlParseFile("-");
 
-        if (!doc) exit(2);
+        if (!doc)
+        {
+            edCleanupNSArr(ns_arr);
+            xmlCleanupParser();
+            xmlCleanupGlobals();
+            exit(2);
+        }
 
         edProcess(doc, ops, ops_count);
 
@@ -1082,13 +1097,21 @@ edMain(int argc, char **argv)
             }
             ret = xmlOutputBufferClose(buf);
         }
+        
+        xmlFreeDoc(doc);
     }
     
     for (n=i; n<argc; n++)
     {
         xmlDocPtr doc = xmlParseFile(argv[n]);
 
-        if (!doc) exit(2);
+        if (!doc)
+        {
+            edCleanupNSArr(ns_arr);
+            xmlCleanupParser();
+            xmlCleanupGlobals();
+            exit(2);
+        }
 
         edProcess(doc, ops, ops_count);
 
@@ -1118,8 +1141,12 @@ edMain(int argc, char **argv)
             }
             ret = xmlOutputBufferClose(buf);
         }
+        
+        xmlFreeDoc(doc);
     }
 
     edCleanupNSArr(ns_arr);
+    xmlCleanupParser();
+    xmlCleanupGlobals();
     return 0;
 }
