@@ -1,10 +1,10 @@
-/*  $Id: xml_validate.c,v 1.30 2004/03/28 21:28:33 mgrouch Exp $  */
+/*  $Id: xml_validate.c,v 1.36 2005/01/07 01:52:43 mgrouch Exp $  */
 
 /*
 
 XMLStarlet: Command Line Toolkit to query/edit/check/transform XML documents
 
-Copyright (c) 2002 Mikhail Grushinskiy.  All Rights Reserved.
+Copyright (c) 2002-2004 Mikhail Grushinskiy.  All Rights Reserved.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -55,6 +55,7 @@ typedef struct _valOptions {
     char *schema;             /* External Schema URL or file name */
     char *relaxng;            /* External Relax-NG Schema URL or file name */
     int   err;                /* Allow stderr messages */
+    int   embed;              /* Validate using embeded DTD */
     int   wellFormed;         /* Check if well formed only */
     int   listGood;           /* >0 list good, <0 list bad */
     int   show_val_res;       /* display file names and valid/invalid message */
@@ -63,7 +64,7 @@ typedef struct _valOptions {
 typedef valOptions *valOptionsPtr;
 
 /*
- * usage string chunk : 509 char min on ISO C90
+ * usage string chunk : 509 char max on ISO C90
  */
 static const char validate_usage_str_1[] =
 "XMLStarlet Toolkit: Validate XML document(s)\n"
@@ -76,11 +77,11 @@ static const char validate_usage_str_1[] =
 static const char validate_usage_str_2[] =
 #ifdef LIBXML_SCHEMAS_ENABLED
 "  -s or --xsd <xsd-file>     - validate against XSD schema\n"
+"  -E or --embed              - validate using embedded DTD\n"
 #endif
 #ifdef LIBXML_SCHEMAS_ENABLED
 "  -r or --relaxng <rng-file> - validate against Relax-NG schema\n"
 #endif
-/*"  -x or --xml-out            - print result as xml\n"*/
 "  -e or --err                - print verbose error messages on stderr\n"
 "  -b or --list-bad           - list only files which do not validate\n"
 "  -g or --list-good          - list only files which validate\n"
@@ -89,7 +90,7 @@ static const char validate_usage_str_2[] =
 #ifdef LIBXML_SCHEMAS_ENABLED
 static const char schema_notice[] =
 "NOTE: XML Schemas are not fully supported yet due to its incomplete\n" 
-"      support in libxml (see http://xmlsoft.org)\n\n";
+"      support in libxml2 (see http://xmlsoft.org)\n\n";
 #endif
 
 /**
@@ -100,12 +101,12 @@ valUsage(int argc, char **argv)
 {
     extern const char more_info[];
     FILE* o = stderr;
-    fprintf(o, validate_usage_str_1);
-    fprintf(o, validate_usage_str_2);
+    fprintf(o, "%s", validate_usage_str_1);
+    fprintf(o, "%s", validate_usage_str_2);
 #ifdef LIBXML_SCHEMAS_ENABLED
-    fprintf(o, schema_notice);
+    fprintf(o, "%s", schema_notice);
 #endif
-    fprintf(o, more_info);
+    fprintf(o, "%s", more_info);
     exit(1);
 }
 
@@ -118,6 +119,7 @@ valInitOptions(valOptionsPtr ops)
     ops->wellFormed = 1;
     ops->listGood = -1;
     ops->err = 0;
+    ops->embed = 0;
     ops->dtd = NULL;
     ops->schema = NULL;
     ops->relaxng = NULL;
@@ -145,6 +147,11 @@ valParseOptions(valOptionsPtr ops, int argc, char **argv)
             ops->err = 1;
             i++;
         }
+        else if (!strcmp(argv[i], "--embed") || !strcmp(argv[i], "-E"))
+        {
+            ops->embed = 1;
+            i++;
+        }
         else if (!strcmp(argv[i], "--list-good") || !strcmp(argv[i], "-g"))
         {
             ops->listGood = 1;
@@ -270,6 +277,7 @@ valAgainstDtd(valOptionsPtr ops, char* dtdvalid, xmlDocPtr doc, char* filename)
                 }
             }
             xmlFreeDtd(dtd);
+            xmlFreeValidCtxt(cvp);
         }
     }
 
@@ -308,6 +316,7 @@ valMain(int argc, char **argv)
         {
             xmlDocPtr doc;
             int ret;
+            int options = 0;
 
             ret = 0;
             doc = NULL;
@@ -319,7 +328,8 @@ valMain(int argc, char **argv)
                 xmlDefaultSAXHandler.warning = NULL;
             }                       
 
-            doc = xmlParseFile(argv[i]);
+            options |= XML_PARSE_DTDVALID;
+            doc = xmlReadFile(argv[i], NULL, options);
             if (doc)
             {
                 /* TODO: precompile DTD once */                
@@ -563,6 +573,54 @@ valMain(int argc, char **argv)
         xmlRelaxNGCleanupTypes();
     }
 #endif
+    else if (ops.embed)
+    {
+        int i;
+        for (i=start; i<argc; i++)
+        {
+            xmlDocPtr doc;
+            int options = 0;
+            int ret;
+
+            ret = 0;
+            doc = NULL;
+
+            if (!ops.err)
+            {
+                xmlDefaultSAXHandlerInit();
+                xmlDefaultSAXHandler.error = NULL;
+                xmlDefaultSAXHandler.warning = NULL;
+            }
+
+            options |= XML_PARSE_DTDVALID;
+            doc = xmlReadFile(argv[i], NULL, options);
+            if (doc != NULL)
+            {
+                if ((ops.listGood > 0) && !ops.show_val_res)
+                {
+                    fprintf(stdout, "%s\n", argv[i]);
+                }
+                xmlFreeDoc(doc);
+            }
+            else
+            {
+                ret = 1; /* Malformed XML or could not open file */
+                if ((ops.listGood < 0) && !ops.show_val_res)
+                {
+                    fprintf(stdout, "%s\n", argv[i]);
+                }
+            }
+            if (ret) invalidFound = 1;
+
+            if (ops.show_val_res)
+            {
+                if (ret == 0)
+                    fprintf(stdout, "%s - valid\n", argv[i]);
+                else
+                    fprintf(stdout, "%s - invalid\n", argv[i]);
+            }
+        }
+    }
     else if (ops.wellFormed)
     {
         int i;
